@@ -1,11 +1,45 @@
+//! ```
+//! use text_align::TextAlign;
+//! assert_eq!(
+//!     "                          Hello my dearest friend!".left_align(50),
+//!     "Hello my dearest friend!                          "
+//! );
+//!
+//! assert_eq!(
+//!     "Hello my dearest friend!".right_align(50),
+//!     "                          Hello my dearest friend!"
+//! );
+//!
+//! assert_eq!(
+//!     "Hello my dearest friend!".center_align(50),
+//!     "             Hello my dearest friend!             "
+//! );
+//!
+//! assert_eq!(
+//!     "Hello my dearest friend!".justify(50),
+//!     "Hello          my          dearest         friend!"
+//! );
+//!
+//! assert_eq!(
+//!     "Hello          my          dearest         friend!".dejustify(2),
+//!     "Hello my dearest friend!"
+//! );
+//! ```
+
 mod helper_functions;
 
 use helper_functions::{get_index_spread, replace_matches};
 use std::fmt;
 
+enum AlignmentType {
+    Center,
+    Left,
+    Right,
+}
+
 pub trait TextAlign {
-    fn center_align(&self, width: usize, should_include_trailing_whitespace: bool) -> String;
-    fn left_align(&self) -> String;
+    fn center_align(&self, width: usize) -> String;
+    fn left_align(&self, width: usize) -> String;
     fn right_align(&self, width: usize) -> String;
     fn justify(&self, width: usize) -> String;
     fn dejustify(&self, spaces_after_punctuation: usize) -> String;
@@ -13,63 +47,16 @@ pub trait TextAlign {
 
 // TODO: Any way to make this return either `&str` or `String`?
 impl<T: AsRef<str> + fmt::Display> TextAlign for T {
-    fn center_align(&self, mut width: usize, should_include_trailing_whitespace: bool) -> String {
-        let mut str_ref = self.as_ref();
-        let text_length = str_ref.len();
-        let has_newline = str_ref.ends_with('\n');
-
-        if has_newline {
-            str_ref = str_ref.trim_end();
-            width += 1;
-        }
-
-        if width <= text_length {
-            return self.to_string();
-        }
-
-        let spaces = width - text_length;
-        let left_padding_length = spaces / 2;
-
-        let right_padding_length = if should_include_trailing_whitespace {
-            spaces - left_padding_length
-        } else {
-            0
-        };
-
-        let last_character = if has_newline { "\n" } else { "" };
-
-        format!(
-            "{}{}{}{}",
-            " ".repeat(left_padding_length),
-            str_ref,
-            " ".repeat(right_padding_length),
-            last_character
-        )
+    fn center_align(&self, width: usize) -> String {
+        align(self, width, AlignmentType::Center)
     }
 
-    // Because left alignment will only ever shorten the length of a line, we dont need to worry
-    // about staying within a width, so we don't even consider it
-    fn left_align(&self) -> String {
-        self.as_ref().trim_start().to_string()
+    fn left_align(&self, width: usize) -> String {
+        align(self, width, AlignmentType::Left)
     }
 
-    fn right_align(&self, mut width: usize) -> String {
-        let str_ref = self.as_ref();
-        let text_length = str_ref.len();
-        let has_newline = str_ref.ends_with('\n');
-
-        if has_newline {
-            width += 1;
-        }
-
-        if width <= text_length {
-            return self.to_string();
-        }
-
-        let padding_length = width - text_length;
-        let padding_string = " ".repeat(padding_length);
-
-        format!("{}{}", padding_string, self)
+    fn right_align(&self, width: usize) -> String {
+        align(self, width, AlignmentType::Right)
     }
 
     fn justify(&self, width: usize) -> String {
@@ -93,7 +80,7 @@ impl<T: AsRef<str> + fmt::Display> TextAlign for T {
 
         let mut space_counts = vec![spaces_per_block; space_blocks_required];
 
-        if let Some(indices) = get_index_spread(space_counts.len(), remaining_spaces) {
+        if let Some(indices) = get_index_spread(remaining_spaces, space_counts.len()) {
             for index in indices {
                 space_counts[index] += 1
             }
@@ -143,6 +130,49 @@ impl<T: AsRef<str> + fmt::Display> TextAlign for T {
     }
 }
 
+fn align<T: AsRef<str> + fmt::Display>(
+    text: T,
+    mut width: usize,
+    alignment_type: AlignmentType,
+) -> String {
+    let mut str_ref = text.as_ref().trim_start();
+    let has_newline = str_ref.ends_with('\n');
+    let text_length = str_ref.len();
+    str_ref = str_ref.trim_end();
+
+    if has_newline {
+        width += 1;
+    }
+
+    if width <= text_length {
+        return text.to_string();
+    }
+
+    let spaces = width - text_length;
+    let last_character = if has_newline { "\n" } else { "" };
+
+    match alignment_type {
+        AlignmentType::Center => {
+            let left_padding_length = spaces / 2;
+            let right_padding_length = spaces - left_padding_length;
+
+            format!(
+                "{}{}{}{}",
+                " ".repeat(left_padding_length),
+                str_ref,
+                " ".repeat(right_padding_length),
+                last_character
+            )
+        }
+        AlignmentType::Left => {
+            format!("{}{}{}", str_ref, " ".repeat(spaces), last_character)
+        }
+        AlignmentType::Right => {
+            format!("{}{}{}", " ".repeat(spaces), str_ref, last_character)
+        }
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -150,47 +180,27 @@ mod tests {
     #[test]
     fn test_center_align_even_length_text() {
         // Non-newline tests
-        assert_eq!("hi".center_align(3, false), "hi");
-        assert_eq!("hi".center_align(3, true), "hi ");
-
-        assert_eq!("hi".center_align(5, false), " hi");
-        assert_eq!("hi".center_align(5, true), " hi  ");
-
-        assert_eq!("hi".center_align(8, false), "   hi");
-        assert_eq!("hi".center_align(8, true), "   hi   ");
+        assert_eq!("hi".center_align(3), "hi ");
+        assert_eq!("hi".center_align(5), " hi  ");
+        assert_eq!("hi".center_align(8), "   hi   ");
 
         // Newline tests
-        assert_eq!("hi\n".center_align(3, false), "hi\n");
-        assert_eq!("hi\n".center_align(3, true), "hi \n");
-
-        assert_eq!("hi\n".center_align(5, false), " hi\n");
-        assert_eq!("hi\n".center_align(5, true), " hi  \n");
-
-        assert_eq!("hi\n".center_align(8, false), "   hi\n");
-        assert_eq!("hi\n".center_align(8, true), "   hi   \n");
+        assert_eq!("hi\n".center_align(3), "hi \n");
+        assert_eq!("hi\n".center_align(5), " hi  \n");
+        assert_eq!("hi\n".center_align(8), "   hi   \n");
     }
 
     #[test]
     fn test_center_align_odd_length_text() {
         // Non-newline tests
-        assert_eq!("doggy".center_align(3, false), "doggy");
-        assert_eq!("doggy".center_align(3, true), "doggy");
-
-        assert_eq!("doggy".center_align(8, false), " doggy");
-        assert_eq!("doggy".center_align(8, true), " doggy  ");
-
-        assert_eq!("doggy".center_align(9, false), "  doggy");
-        assert_eq!("doggy".center_align(9, true), "  doggy  ");
+        assert_eq!("doggy".center_align(3), "doggy");
+        assert_eq!("doggy".center_align(8), " doggy  ");
+        assert_eq!("doggy".center_align(9), "  doggy  ");
 
         // Newline tests
-        assert_eq!("doggy\n".center_align(3, false), "doggy\n");
-        assert_eq!("doggy\n".center_align(3, true), "doggy\n");
-
-        assert_eq!("doggy\n".center_align(8, false), " doggy\n");
-        assert_eq!("doggy\n".center_align(8, true), " doggy  \n");
-
-        assert_eq!("doggy\n".center_align(9, false), "  doggy\n");
-        assert_eq!("doggy\n".center_align(9, true), "  doggy  \n");
+        assert_eq!("doggy\n".center_align(3), "doggy\n");
+        assert_eq!("doggy\n".center_align(8), " doggy  \n");
+        assert_eq!("doggy\n".center_align(9), "  doggy  \n");
     }
 
     #[test]
@@ -209,12 +219,14 @@ mod tests {
     #[test]
     fn test_left_align() {
         // Non-newline tests
-        assert_eq!("hi".left_align(), "hi");
-        assert_eq!(" hi".left_align(), "hi");
+        assert_eq!("hi".left_align(1), "hi");
+        assert_eq!("hi".left_align(3), "hi ");
+        assert_eq!(" hi".left_align(5), "hi   ");
 
         // Newline tests
-        assert_eq!("hi\n".left_align(), "hi\n");
-        assert_eq!(" hi\n".left_align(), "hi\n");
+        assert_eq!("hi\n".left_align(1), "hi\n");
+        assert_eq!(" hi\n".left_align(3), "hi \n");
+        assert_eq!(" hi\n".left_align(5), "hi   \n");
     }
 
     #[test]
